/**
 * 商品加入购物车或修改数量，调出购物车数量编辑框，调用时使用editCart.show方法，带JSON参数；
 * 删除购物车商品，使用editCart.deleteCartId方法，带参数
 *
 */

var editCart = {

    /*
     * 生成购物车数量编辑控件显示
     * show参数为JSON格式
     * {
     *
     *   merchandiseId: ,     @merchandiseId {Number} 所购商品ID
     *   callback:,           @callback {String} 添加完成后回调的js脚本方法名，ajax访问的result转为str后作为该脚本函数的参数
     *   popName:,            @popName {String}  执行回调函数的浮动窗口名称
     *
     *   winName:,            @winName {String}  非必需，执行回调函数的浮动窗口所在主窗口名称，为空则为当前窗口
     *   oldNum:,             @oldNum {Number}   非必需，购物车中原购买数量,不带或为空则为新增，非空则为更新
     *   fromCartPage,       @fromCartPage{boolean} 非必需，如果从购物车明细页面本身显示购物车数量添加界面，
     *                                      则不需要选择客户，否则需要先选择客户才能加入购物车
     *   inPageName,          @inPageName     非必需，如果执行回调函数的窗口为多页浮动窗口，则传入本参数
     *
     *
     *    //新补充
     *   isDecimal:             0/1       是否可输入小数
     *   retail                 0/1          是否可拆零
     *   middlingPacking：             { Number }    中包装数  不可拆零的产品，传入中包装数，购物车按钮增加时，每次增加数为中包装数
     * }
     */

    show : function(jsonPara) {
        if (!jsonPara.fromCartPage) {
            if (!LocStorage.get('orderCust')) {
                open({
                    name : 'Cust_Location',
                    data : 'Search/Cust.html?name=OutsideBusinessApp&popName=content&inPageName=content_0&scriptContent=setCust()'
                });
                return;
            }
        }
        if (jsonPara.merchandiseId) {
            var id = jsonPara.merchandiseId;
            jsonPara.merchandiseId = parseInt(id);
        }
        var para = JSON.stringify(jsonPara);
        LocStorage.set('shoppingCartPara', para);

        appcan.frame.open({
            id : 'shoppingCart',
            url : 'shoppingCart.html',
        });
    },

    ok : function(para) {
        //log('OKpara');
        //log(para);
        var isAdd = true;
        var supplierCustId;

        //判断该操作为新增还是更新
        var UserAgent = JSON.parse(LocStorage.get('UserAgent'));
        var userSupplierId = UserAgent.userSupplierId;
        if (para.fromCartPage) {
            supplierCustId = LocStorage.get('cartDetail_supplierCustId');
        } else {
            var chooseCust = LocStorage.get('orderCust');
            var cust = JSON.parse(chooseCust);
            supplierCustId = cust.supplierCustId;
        }
        var data = [{
            merchandiseId : para.merchandiseId,
            merchandiseNumber : para.newNum,
        }];

        if (para.activityType) {
            data[0].activityType = para.activityType;
        }

        if (para.orgMerchandiseCode) {
            data[0].orgMerchandiseCode = para.orgMerchandiseCode;
        }

        var array = JSON.stringify(data);
        //array=array+'&userSupplierId='+userSupplierId+'&supplierCustId='+supplierCustId;
        var param = {
            array : array,
            userSupplierId : userSupplierId,
            supplierCustId : supplierCustId
        };
        if (parseFloat(para.oldNum)) {
            isAdd = false;
            param.blurCart = 'blurCart';
            param.cartId = para.cartId;
        }
        AjaxFn(urls.CART_ADD, param, function(result) {
            //log('加入購物車返回' + JSON.stringify(result));
            //log('para參數' + JSON.stringify(para));

            var name = para.winName || '';
            var popName = para.popName;
            var scriptContent = para.callback + '(' + JSON.stringify(result) + ')';
            if (para.oldNum) {
                scriptContent = para.callback + '(' + JSON.stringify(result) + ',' + para.newNum + ')';
            }
            if (para.inPageName) {
                uexWindow.evaluateMultiPopoverScript(name, popName, para.inPageName, scriptContent);

            } else if (para.popName) {
                appcan.frame.evaluateScript({
                    name : para.winName || '',
                    popName : para.popName,
                    scriptContent : scriptContent
                });
            } else {
                appcan.window.evaluateScript({
                    name : para.winName || '',
                    scriptContent : scriptContent
                });
            }

            uexWindow.evaluateMultiPopoverScript('OutsideBusinessApp', 'content', 'content_1', 'initPage();');
        }, {
            successException : function(result) {
                editCart.successException = true;
                var msg = result.msgInfo || result.msg|| result.message || '加入购物车失败，请联系管理员！';
                if (result.conditionone) {
                    msg = result[result.conditionone] || msg;
                }
                alert(msg, {
                    callback : function(opId, dataType, data) {
                        //log(data);
                        if (data == 0) {
                            appcan.window.close(-1);
                        }
                    }
                });
            },
            completeFn : function() {
                if (!editCart.successException) {
                    appcan.window.close(-1);
                }
            },
            loadView:true
        });

    },

    /*
     * 删除购物车中的商品，
     * @cartId {String} 多个所需删除的商品id，用逗号分隔，例如'242344,451133,5513423,'
     * @callback {Function}  添加完成后回调函数
     *
     */
    deleteCartId : function(cartId, callback) {
        AjaxFn(urls.CART_DELETE, {
            cartId : cartId
        }, callback);
    },
};
