/**
 * 商品加入购物车或修改数量，调出购物车数量编辑框，调用时使用editCart.show方法，带JSON参数；
 * 删除购物车商品，使用editCart.deleteCartId方法，带参数
 *
 */

var editCart = {

    /*
     * 生成购物车数量编辑控件显示
     * show参数为JSON格式
     * {
     *   merchandiseId: ,     @merchandiseId {Number} 所购商品ID，如存在group参数则此参数可不传
     *   callback:,           @callback {String} 添加完成后回调的js脚本方法名，ajax访问的result转为str后作为该脚本函数的参数
     *
     *   popName:,            @popName {String}  非必需，执行回调函数的浮动窗口名称，若为空则在主窗口执行脚本
     *   winName:,            @winName {String}  非必需，执行回调函数的浮动窗口所在主窗口名称，若为空则为当前窗口
     *   inPageName,          @inPageName {String}  非必需，如果执行回调函数的窗口为多页浮动窗口，    则传入本参数，此时也必须传入popName和winName
     *
     *   //关于拆零和小数购买，非必需，不传则默认为：不可输入小数，可拆零
     *   isDecimal:             0/1         购买数量是否可输入小数
     *   retail                 0/1         是否可拆零
     *   middlingPacking：             { Number }    中包装数  不可拆零的产品，传入中包装数，购物车按钮增加时，每次增加数为中包装数
     *
     *   //限制购买数量
     *   lowestAmount: ,     @lowestAmount {Number}  非必需,最低购买数
     *   highestAmount:      @highestAmount {Number} 非必需，最高购买数
     *
     *  //其他数据
     *   oldNum:,             @oldNum {Number}   非必需，调出购物车时，购物车默认采购数量    
     *   isUpdate:,           @isUpdate{Boolean} 非必需，不带、为空、false则为新增，true为更新
     *   
     *  //精品组合数据
     *   group: ,             @group{Array}      非必需，用于精品组合加入购物车时使用，格式为[{merchandiseId : 201826939, merchandiseNumber : 3,activityGroupId:2},…]
     *   groupLowData:        @groupLowData{Array}   非必需，用于精品组合加入购物车时采购数量限制使用，标识一套组合中的每个产品数量（即最低采购数量），格式[12,5,10]
     *   groupHighData:       @groupHighData{Array}  非必需，用于精品组合加入购物车时采购数量限制使用，标识每个产品的最高采购数量格式[48,20,40]
     * }

     */

    show : function(jsonPara) {
        if (jsonPara.merchandiseId) {
            var id = jsonPara.merchandiseId;
            jsonPara.merchandiseId = parseInt(id);
        }
        var para = JSON.stringify(jsonPara);
        LocStorage.set('shoppingCartPara', para);

        appcan.frame.open({
            id : 'shoppingCart',
            url : 'shoppingCart.html',
        });

    },

    ok : function(para) {
       
        var isUpdate = para.isUpdate;
        var data = [];
        if (para.group) {
            data = para.group;
        } else {
            data = [{
                merchandiseId : para.merchandiseId,
                merchandiseNumber : para.newNum
            }];
        }

        if (para.activityType) {
            data[0].activityType = para.activityType;
        }

        if (para.orgMerchandiseCode) {
            data[0].orgMerchandiseCode = para.orgMerchandiseCode;
        }

        var array = JSON.stringify(data);
        var requestPara = {
            array : array
        };
        if (isUpdate) {          
            requestPara.blurCart = 'blurCart';
        }

        AjaxFn(urls.CART_ADD, requestPara, function(result) {
            if (!isUpdate) {
                if(isPhone){
                    uexWindow.evaluateMultiPopoverScript('B2BApp', 'content', 'content_2', 'updatePage(true);');
                }
                
                //新增购物车品种刷新购物车角标数量-------------luqq
                appcan.window.evaluateScript({
                    name : 'B2BApp',
                    scriptContent : 'getCartNum();'
                });
            }
            var name = para.winName || '';
            var popName = para.popName;
            var scriptContent = para.callback + '(' + JSON.stringify(result) + ')';
            if (para.oldNum) {
                scriptContent = para.callback + '(' + JSON.stringify(result) + ',' + para.newNum + ')';
            }
            if (para.inPageName) {
                uexWindow.evaluateMultiPopoverScript(name, popName, para.inPageName, scriptContent);
            } else if (para.popName) {
                appcan.frame.evaluateScript({
                    name : name,
                    popName : popName,
                    scriptContent : scriptContent
                });
            } else {
                appcan.window.evaluateScript({
                    name : name,
                    scriptContent : scriptContent
                });
            }

        }, {
            successException : function(result) {
                editCart.successException = true;
                var msg=result.msgInfo || result.msg||result.message||'加入购物车失败，请联系管理员！';
                if(result.conditionone){
                    msg=result[result.conditionone]||msg;
                }
                alert(msg, {
                    callback : function(opId, dataType, data) {
                        if (data == 0) {
                            appcan.window.close(-1);
                        }
                    }
                });
            },
            completeFn : function() {
                if (!editCart.successException) {
                    appcan.window.close(-1);
                }
            },
            loadView:true
        });

    },

    /*
     * 删除购物车中的商品，
     * @cartId {String} 多个所需删除的商品id，用逗号分隔，例如'242344,451133,5513423,'
     * @callback {Function}  添加完成后回调函数
     *
     */
    deleteCartId : function(cartId, callback) {
        AjaxFn(urls.CART_DELETE, {
            cartId : cartId
        }, callback);
    },
};
