appcan.define("treeview", function($, exports, module) {

    var listTmp ='<li class="treeview">\
        <div class="treeview-header ubb bc-border bc-text ub uinn ub-ac">\
        <%if (option.hasIcon) {%>\
        <div class="lis-icon-s ub-img" style="background-image:url(<%=icon%>)"></div>\
        <%}%>\
        <div class="ub-f1 ut-s"><label class="ub ub-ac"><div class="checkbox"><input type="checkbox" class="uabs ub-con"></div><div class="ub-f1"><%=header%></div></label></div>\
        <%if(option.hasAngle){%>\
        <div class="fa fa-angle-up ulev2 utra sc-text"></div>\
        <%}%>\
        </div>\
        <div class="treeview-content bc-text">\
        <%=content%>\
        </div>\
        </li>';
        
    var listRender = appcan.view.template(listTmp);

    var isEmulator = !('ontouchstart' in window);

    var touchEvent = {
        'start' : isEmulator ? 'mousedown' : 'touchstart',
        'move' : isEmulator ? 'mousemove' : 'touchmove',
        'end' : isEmulator ? 'mouseup' : 'touchend',
        'cancel' : 'touchcancel'
    };


    var Treeview = function(option) {
        appcan.extend(this, appcan.eventEmitter);
        this.option = $.extend({
            selector : "body",
            type : '',
            hasIcon : false,
            hasAngle : true,
            hasTouchEffect : true,
            touchClass : 'sc-bg-active',
            isCloseOther : true,
            defaultOpen : -1,
            autoScrollTop : false
        }, option, true);
        this.ele = $(this.option.selector);
        if (this.option.data) {
            this.set(this.option.data);
        }
        this.open(this.option.defaultOpen);
    };

    Treeview.prototype = {
        constructor : Treeview,
        createPlugin : function(selector) {
            if (!this.plugin) {
                return;
            }
            var option = $.extend({
                selector : selector
            }, this.pluginOption, true);
            return this.plugin(option)
        },
        getContent : function(data, settings) {
            var contentData = data.content;
            if (!contentData) {
                return;
            }
            var self = this;
            var index = 0;
            if (!settings.type) {
                if ((appcan.isArray(contentData) || appcan.isPlainObject(contentData)) && !$.zepto.isZ(contentData)) {
                    $.each(contentData,function(index,item){
                        item.title='<label class="ub ub-ac"><div class="checkbox"><input type="checkbox" class="uabs ub-con"></div><div class="ub-f1">'+item.header+'</div></label>';
                    });
                    var warper = $('<div class="listview"></div>');
                    settings.option.selector = warper;
                    settings.option.hasAngle=contentData.content?true:false;
                    var lv = appcan.listview(settings.option);
                    lv.set(contentData);
                    warper[0].plugin = lv;
                    return warper;
                }
                return contentData;
            } else {
                var plugin = appcan[settings.type];
                if (plugin) {
                    settings.option.data = contentData;
                    return plugin(settings.option);
                }
            }
        },
        buildTreeview : function(data, settings) {
            var container = $("<ul></ul>");
            var self = this;
            for (var i in data) {
                var html = listRender;
                var option = {};
                option = $.extend({}, settings && settings.option, true);
                option = $.extend(option, data[i].option, true);
                option.hasTouchEffect=false;
                var processType = data[i].type || (settings && settings.type);
                settings = {
                    option : option,
                    type : processType              
                };
                var content = self.getContent(data[i], settings);
                var ele = $(html({
                    header : data[i].header,
                    icon:data[i].icon,
                    content : '',
                    option : self.option
                }));
                var tvHeader = ele.find('.treeview-header');
                tvHeader[0]["tv_data"] = data[i];
                if (!settings.type) {
                    if (content) {
                        ele.find('.treeview-content').html(content);
                    } else {
                        ele.find('.treeview-header .fa-angle-up').removeClass('fa-angle-up');
                        //ele.find('.treeview-header .fa-angle-up').removeClass('fa-angle-up').addClass('fa-angle-right');
                        ele.find('.treeview-content').remove();
                    }
                } else {
                    ele[0]['plugin'] = content;
                }
                tvHeader.on('tap', function(evt) {
                    self.itemClick(evt);
                });                 
                container.append(ele);
            }
            container.find('input[type="checkbox"]').on('change',function(evt){
                    self.treeItemCheckedChange(evt);
                });
            
            self.on('dbclick', function(ele, obj, target,checked) {
                     self.emit("treeItemCheckedChange",'',self, ele, obj, target,checked);
                });
            
            return container;
        },
        set : function(data, settings) {
            var self = this;
            var container = self.buildTreeview(data, settings);
            self.ele.html(container);
            return self;
        },
        add : function(data, dir, settings) {
            var self = this;
            var container = self.buildTreeview(data, settings);
            if (dir || dir == 1) {
                self.ele.append(container)
            } else {
                var first = self.ele.children().first();
                first.prepend(container);
            };
            return self;
        },
        treeItemCheckedChange:function(evt){
            var obj = $(evt.currentTarget);
            var tvHeader = obj.parents('.treeview-header'),data;
            if(tvHeader.length==0){
                 tvHeader = obj.parents('.lis[data-index]');
                 data=tvHeader[0]["lv_data"];
            }
            else{
                data=tvHeader[0]["tv_data"];
            }
            this.emit("dbclick",  tvHeader,data , $(evt.target),obj[0].checked);   
        },
        itemClick : function(evt) {
            var target=$(evt.target);
            if(target.hasClass('fa-angle-up')){
                
            var treeviewHeader = $(evt.currentTarget);
            var data = treeviewHeader[0]["tv_data"];
            if(data.content){
                this.showItem(treeviewHeader);}
            }
        },
        showItem : function(header) {
            var treeviewHeader = header;
            var obj = treeviewHeader.parent();

            var contentEle = obj.find('.treeview-content');
            if(treeviewHeader.length){
                var data = treeviewHeader[0]["tv_data"];
                if(data.content)
                    obj.siblings().find('.treeview-header').removeClass(this.option.touchClass);
            }
            if (this.option.isCloseOther) {
                obj.siblings().find('.treeview-content').addClass('uhide');
                obj.siblings().find('.fa-angle-up').addClass('fa-rotate-180');
            }
            if (contentEle.hasClass('uhide')) {
                contentEle.removeClass('uhide');
                treeviewHeader.addClass(this.option.touchClass);
                obj.find('.fa-angle-up').removeClass('fa-rotate-180');
            } else {
                contentEle.addClass('uhide');
                treeviewHeader.removeClass(this.option.touchClass);
                obj.find('.fa-angle-up').addClass('fa-rotate-180');
            }
            if (this.option.autoScrollTop) {
                var offset = treeviewHeader.offset();
                var top = offset && offset.top || 0;
                $(window).scrollTop(top);
            }
        },
        //打开索引位置的选项
        open : function(index) {
            index = parseInt(index, 10);
            if (isNaN(index) || index < 0) {
                return;
            }
            this.showItem(this.ele.find('.treeview').eq(index).find('.treeview-header'));
        }
    };

    module.exports = function(option) {
        return new Treeview(option);
    };
});
